@@ -1,7 +1,7 @@
 /* The name and version of this software distribution. These
  * originates from AM_INIT_AUTOMAKE in the configure.in file. */
-#define PACKAGE "configure-generated"
-#define VERSION "configure-generated"
+#undef PACKAGE
+#undef VERSION
 
 /* Define to enable sanity checking on memory allocation and casting */
 #undef DEBUG_ALLOC
