@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: io.h,v 1.50 1999/11/28 18:15:56 nisse Exp $ */
+ * $Id: io.h,v 1.51 1999/12/07 00:37:25 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -371,4 +371,9 @@ struct io_fd *io_write_file(struct io_backend *backend,
 			    struct close_callback *c,
 			    struct exception_handler *e);
 
+struct io_fd *
+io_read_file(struct io_backend *backend,
+	     const char *fname, 
+	     struct exception_handler *e);
+
 #endif /* LSH_IO_H_INCLUDED */
